/*
  NetLoadModule LiteStep Module - version 2.4.0

  Copyright (C) 2002 Joshua Seagoe
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Remember to fix the include paths and link with lsapi.lib

#include "NetLoadModule.h"

#define LS_SUCCESS 0
#define LS_ERROR   1

extern "C" {
__declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath);
__declspec( dllexport ) int quitModule(HINSTANCE dll);
}

s_GlobalData GlobalData;



/* actual parsers are in parsers.cpp and lsmod_common.cpp */ 
void FreeSiteProc(void *p,LPARAM);
bool ParseSiteProc(void *p,const _TCHAR *line,LPARAM);
void FreeModuleProc(void *p,LPARAM);

inline int ParseSites(void)
{
	return (GlobalData.nSites =
		ParseStarList(CONF_SITE,CONF_SITE_LEN,ParseSiteProc,&GlobalData.pSites,sizeof(s_Site),NULL));
}

inline void FreeSites(void)
{
	FreeStarList(FreeSiteProc,GlobalData.pSites,NULL);
}

inline int ParseModules(void)
{
	return
		ParseStarList(CONF_MODULE,CONF_MODULE_LEN,ParseModuleProc,&GlobalData.pModules,sizeof(s_Module),PARSE_LOAD_MODULE)
    + ParseStarList(CONF_INSTALL,CONF_INSTALL_LEN,ParseModuleProc,&GlobalData.pModules,sizeof(s_Module),PARSE_INSTALL_MODULE);
}

inline void FreeModules(void)
{
	FreeStarList(FreeModuleProc,GlobalData.pModules,NULL);
}


/*
      InstallModules

  Not to be confused with InstallModule.
  Make sure all of the modules are installed.
  This will do nothing if all of the modules
  are already here.
*/
void InstallModules(void)
{
	if (GlobalData.bAnyFail || GlobalData.bTestMessage)
	{
    /* this stuff only needs to be done once */ 
    if (!GlobalData.bDownloadInitDone)
    {
		  ParseSites();

		  if (GlobalData.nSites<1)
		  {
        _TCHAR message[NLM_PATH_LENGTH];
	      LoadString(GlobalData.hInstance,IDS_NODOWNLOADSITES,message,NLM_PATH_LENGTH);
			  MessageBox(NULL,message,TITLE,ERROR_MESSAGE_FLAGS);
		  }
		  else if (GlobalData.nSites>1)
		  {
			  BalanceSites();
		  }

	    INITCOMMONCONTROLSEX cc;
	    cc.dwSize = sizeof(cc);
	    cc.dwICC = ICC_LISTVIEW_CLASSES|ICC_PROGRESS_CLASS;
		  InitCommonControlsEx(&cc);

      GlobalData.bDownloadInitDone = true;
    }

		/* bring up the "modules not found" dialog box  */ 
		DialogBox(GlobalData.hInstance,MAKEINTRESOURCE(IDD_MODULE_LIST),NULL,ModListDlgProc);
	}
}

/*
			LoadModules

	Loop through all the modules, sending a
	LM_RELOADMODULE message to litestep to 
	load each one.
*/
bool LoadModules(void)
{
	s_Module *pMod;
  bool bAnyFailLoad = false;

	pMod = GlobalData.pModules;
	while (pMod)
	{
		ASSERT(pMod->DllPath!=NULL);

		if (!pMod->bDontLoad)
    {
      if (pMod->Exists)
			  SendMessage(GlobalData.hLiteStepWnd,
          LM_RELOADMODULE,
          (WPARAM)pMod->DllPath,
          (LPARAM)pMod->Flags);
      else
        bAnyFailLoad = true;
    }

		pMod = pMod->pNext;
	}

  return !bAnyFailLoad;
}

/*
      ProcessModuleList

  This installs, loads, and frees the module
  list created by ParseModules at startup or
  any time later by bang commands.
*/
bool ProcessModuleList(void)
{
  bool bLoadOK;

  InstallModules();

	bLoadOK = LoadModules();

	FreeModules();

  return bLoadOK;
}



/*
		initModuleEx

*/
int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath)
{
	TRACE(TITLE);
	START_TRACE;

	/* initialize all global data to zero     */ 
	/* this makes sure all pointers are NULL  */ 
	/* and all bools are false                */ 
	ZeroMemory(&GlobalData,sizeof(s_GlobalData));

	GlobalData.hInstance = dll;
	GlobalData.hLiteStepWnd = GetLitestepWnd();

	_TCHAR buffer[NLM_PATH_LENGTH];

	/* get litestep path                      */ 
	if (!GetRCString(CONF_LITESTEPDIR,buffer,"",NLM_PATH_LENGTH))
	{
		TRACE("GetRCString(LiteStepDir) failed");
		VarExpansion(buffer,PATH_LITESTEPDIR);
	}
	GlobalData.LiteStepPathLen = _tcslen(buffer);
	GlobalData.LiteStepPath = new _TCHAR[GlobalData.LiteStepPathLen+1];
	_tcscpy(GlobalData.LiteStepPath,buffer);

	/* get global alias list                  */ 
	if (!GetRCString(CONF_ALIASFILE,buffer,"",NLM_PATH_LENGTH))
	{
		if (GlobalData.LiteStepPathLen+PATH_ALIASFILE_LEN < NLM_PATH_LENGTH)
		{
			_tcscpy(buffer,GlobalData.LiteStepPath);
			_tcscpy(buffer+GlobalData.LiteStepPathLen,PATH_ALIASFILE);
		}
		else
		{
			/* paths aren't allowed to be this long, */ 
			/* so it will never happen               */ 
			ASSERT(0);
			TRACE("Alias file path overflow.");
		}
	}
	GlobalData.AliasFile = new _TCHAR[_tcslen(buffer)+1];
	_tcscpy(GlobalData.AliasFile,buffer);

	/* read the module path                   */ 
	if (!GetRCString(CONF_MODULEDIR,buffer,"",NLM_PATH_LENGTH))
	{
		if (GlobalData.LiteStepPathLen+PATH_MODULEDIR_LEN < NLM_PATH_LENGTH)
		{
			_tcscpy(buffer,GlobalData.LiteStepPath);
			_tcscpy(buffer+GlobalData.LiteStepPathLen,PATH_MODULEDIR);
		}
		else
		{
			/* paths aren't allowed to be this long, */ 
			/* so it will never happen               */ 
			ASSERT(0);
			TRACE("Module path overflow.");
		}
	}
	if (!*buffer) return LS_ERROR; /* must have a module dir */ 

	GlobalData.ModulePathLen = _tcslen(buffer);
	GlobalData.ModulePath = new _TCHAR[GlobalData.ModulePathLen+1];
	_tcscpy(GlobalData.ModulePath,buffer);
	/* create dir if it's not there  */ 
	CreateDirectory(GlobalData.ModulePath,NULL);

	/* read the documentation path            */ 
	if (!GetRCString(CONF_DOCDIR,buffer,"",NLM_PATH_LENGTH))
	{
		if (GlobalData.ModulePathLen+PATH_DOCDIR_LEN < NLM_PATH_LENGTH)
		{
			_tcscpy(buffer,GlobalData.ModulePath);
			_tcscpy(buffer+GlobalData.ModulePathLen,PATH_DOCDIR);
		}
		else
		{
			/* paths aren't allowed to be this long, */ 
			/* so it will never happen               */ 
			ASSERT(0);
			TRACE("Doc path overflow.");
		}
	}
	/* empty path = don't store documentation */ 
	if (*buffer)
	{
		GlobalData.DocPathLen = _tcslen(buffer);
		GlobalData.DocPath = new _TCHAR[GlobalData.DocPathLen+1];
		_tcscpy(GlobalData.DocPath,buffer);
		/* create dir if it's not there  */ 
		CreateDirectory(GlobalData.DocPath,NULL);
	}

	/* read the path to store zip files       */ 
	if (GetRCString(CONF_ZIPDIR,buffer,"",NLM_PATH_LENGTH)
	/* empty path = don't store zip files     */ 
		&& *buffer)
	{
		GlobalData.ZipPathLen = _tcslen(buffer);
		GlobalData.ZipPath = new _TCHAR[GlobalData.ZipPathLen+1];
		_tcscpy(GlobalData.ZipPath,buffer);
		/* create dir if it's not there  */ 
		CreateDirectory(GlobalData.ZipPath,NULL);
	}

	GlobalData.bTestMessage = FALSE!=GetRCBool(CONF_TEST,TRUE);

  RegisterBangs(Bangs);

	ParseModules();

  if (ProcessModuleList())
  {
  	if (GetRCLine(CONF_EVENT_OK,buffer,NLM_PATH_LENGTH,""))
      LSExecute(GlobalData.hLiteStepWnd, buffer, SW_SHOWNORMAL);
  }
  else
  {
  	if (GetRCLine(CONF_EVENT_FAIL,buffer,NLM_PATH_LENGTH,""))
      LSExecute(GlobalData.hLiteStepWnd, buffer, SW_SHOWNORMAL);
  }

	return LS_SUCCESS;
}



int quitModule(HINSTANCE dll)
{
	/* don't unload the modules,          */ 
	/* litestep takes care of that for us */ 

  RemoveBangs(Bangs);

	FreeSites();

	if (GlobalData.LiteStepPath) delete[] GlobalData.LiteStepPath;
	if (GlobalData.AliasFile)    delete[] GlobalData.AliasFile;
	if (GlobalData.DocPath)      delete[] GlobalData.DocPath;
	if (GlobalData.ModulePath)   delete[] GlobalData.ModulePath;
	if (GlobalData.ZipPath)      delete[] GlobalData.ZipPath;

	return LS_SUCCESS;
}

#define RC_HASH_IMPL 1
#include "hash.h"
