
#include "NetLoadModule.h"
#include "keywords.h"

#ifndef NO_SCRIPTING

/* this really should be a hash table rather than  */ 
/* a linked list, but I don't have time right now  */ 
/* and I don't think O(N) lookup times will really */ 
/* matter much (N is expected to be small)         */ 
struct s_File
{
  s_File *pNext;
  TCHAR *name;
  bool bLoad;
};

struct s_InstallScript
{
  s_Module *Module;
  struct s_Install
  {
    s_File *Files;
    bool bInFolder;
  };
  s_Install iModule, iDocs;
};

struct s_ScriptFile
{
  TCHAR Line[NLM_PATH_LENGTH];
  TCHAR Token[NLM_PATH_LENGTH];
  TCHAR *Script;
  const TCHAR *CurPos;
  s_InstallScript *p;
  s_InstallScript::s_Install *pInstall;
};

static bool GetScriptLine(s_ScriptFile *p)
{
  TCHAR *l = p->Line;
  while (*p->CurPos=='\n' || *p->CurPos=='\r')
    ++p->CurPos;
  if (!*p->CurPos) return false;
  while (*p->CurPos && *p->CurPos!='\n' && *p->CurPos!='\r')
    *l++ = *p->CurPos++;
  *l = 0;
  return true;
}

/* cheesy template-based finite state machine */ 

class InstallParserState
{
public:
  virtual InstallParserState *operator()
    (int nCmd, s_ScriptFile *f, const TCHAR *params)=0;
};

template <int curstate>
class InstallParserState_t : public InstallParserState
{
public:
  static InstallParserState_t<curstate> S;
  InstallParserState *operator()(int nCmd, s_ScriptFile *f, const TCHAR *params);
};

/* I do not want to type all of that. */ 
#define DEFINE_STATE(ST) \
  InstallParserState_t<ST> InstallParserState_t<ST>::S; \
  InstallParserState *InstallParserState_t<ST>::operator () \
    (int nCmd, s_ScriptFile *f, const TCHAR *sParams)
#define STATE(ST) &InstallParserState_t<ST>::S
#define GET_PARAM() \
    if (!GetToken(sParams, f->Token, &sParams, FALSE)) \
      return NULL; else

DEFINE_STATE(0)
{
  switch(nCmd)
  {
  case SCRIPT_SIGNATURE:
    GET_PARAM();
    if (atoi(f->Token) != SCRIPT_MAJOR_VERSION)
      return NULL;
    GET_PARAM();
    if (atoi(f->Token) > SCRIPT_MINOR_VERSION)
      return NULL;
    return STATE(1);
  default: return NULL;
  }
}

DEFINE_STATE(1)
{
  switch(nCmd)
  {
  case SCRIPT_MODULE:
    GET_PARAM();
    if (lstrcmp(f->p->Module->Name, f->Token) != 0)
      return NULL;
    return STATE(2);
  default: return NULL;
  }
}

DEFINE_STATE(2)
{
  switch(nCmd)
  {
  case SCRIPT_OPEN: return STATE(3);
  case SCRIPT_INSTALL:
    GET_PARAM();
    try
    {
      switch(InstallKeywords[f->Token])
      {
      case SCRIPT_MODULE:
        f->pInstall = &f->p->iModule;
        return STATE(4);
      case SCRIPT_DOCS:
        f->pInstall = &f->p->iDocs;
        return STATE(4);
      }
    } catch(...) { }
    return NULL;
  default: return NULL;
  }
}

/* module options */ 
DEFINE_STATE(3)
{
  switch(nCmd)
  {
  case SCRIPT_CLOSE: return STATE(2);
  default: return NULL;
  }
}

/* inside install block */ 
DEFINE_STATE(4)
{
  bool bLoadFile = false;

  switch(nCmd)
  {
  case SCRIPT_OPEN: return STATE(5);
  case SCRIPT_CLOSE: return STATE(2);
  case SCRIPT_LOAD:
    bLoadFile = true;
  case SCRIPT_FILE:
    GET_PARAM();
    {
      s_File *pFile = new s_File;
      pFile->bLoad = bLoadFile;
      pFile->name = new TCHAR[lstrlen(f->Token)+1];
      lstrcpy(pFile->name, f->Token);
      pFile->pNext = f->pInstall->Files;
      f->pInstall->Files = pFile;
    }
    return STATE(4);
  case SCRIPT_OPTION:
    GET_PARAM();
    try
    {
      switch(InstallKeywords[f->Token])
      {
      case SCRIPT_NORENAME:
        /* avoid renaming by installing in a subdirectory  */ 
        /* this is the default if there are multiple files */ 
        f->pInstall->bInFolder = true;
        return STATE(4);
      }
    } catch(...) { }
    return NULL;
  default: return NULL;
  }
}

/* file options */ 
DEFINE_STATE(5)
{
  switch(nCmd)
  {
  case SCRIPT_CLOSE: return STATE(4);
  default: return NULL;
  }
}

bool ParseScript(s_ScriptFile *f)
{
  const TCHAR *sParam;
  int nCmd;
  bool bSignatureOK = false;
  InstallParserState *pState = STATE(0);

  while(GetScriptLine(f))
  {
    if (!GetToken(f->Line, f->Token, &sParam, FALSE))
      continue; /* skip blank lines */ 
    if (*f->Token == '#') continue; /* skip comments */ 

    /* should have an operator() on the hash class */ 
    /* that takes a default value, so I can avoid  */ 
    /* all of this exception handling crud.        */ 
    /* A default for the entire hash class might   */ 
    /* also be neat, but that would involve        */ 
    /* modifying makehash.                         */ 
    try { nCmd = InstallKeywords[f->Token]; }
    catch(...) { nCmd = SCRIPT_UNKNOWN; }

    pState = (*pState)(nCmd, f, sParam);
    if (!pState) return false;
  }
  return true;
}

#endif // NO_SCRIPTING